from __future__ import with_statement
from __future__ import absolute_import
from __future__ import division


import math
from vector2d import Vector2D as vec2
from OpenGL.GL import *
from lightshader import LightShader


shader = None
def get_shader():
    global shader
    if shader:
        return shader
    shader = LightShader()
    shader.create_program()
    return shader


class Light():
    def __init__(self, position, radius=600, depth=0.0, color=(1.0, 1.0, 1.0)):
        self.position = position
        self.depth = depth
        self.radius = radius
        self.source_radius = 5

        import random
        self.color = [random.randint(100, 255)/255,
                random.randint(100, 255)/255, random.randint(100, 255)/255]

    def set_position(self, position):
        self.position = position

    def outer_vector(self, edge, step):
        cv = vec2((self.position[0] - edge[0]), \
                (self.position[1] - edge[1]))

        use_negative = False
        if self.position[0] < edge[0]:
            use_negative = True

        perp_vec = vec2((self.position[0] - edge[0]), \
                (self.position[1] - edge[1]))
        perp_vec.normalize()
        if step == 1:
            if use_negative:
                perp_vec *= -self.source_radius
                perp_vec.rotate((math.pi*2/4))
            else:
                perp_vec *= self.source_radius
                perp_vec.rotate(-(math.pi*2/4))
        else:
            if use_negative:
                perp_vec *= -self.source_radius
                perp_vec.rotate(-(math.pi*2/4))
            else:
                perp_vec.rotate((math.pi*2/4))
                perp_vec *= self.source_radius
        cv = vec2((self.position[0] + perp_vec[0]) - edge[0], (self.position[1] + perp_vec[1]) - edge[1])
        cv *= -1

        cv.normalize()
        return cv * self.radius * 10

    def inner_vector(self, edge, step):
        cv = vec2((self.position[0] - edge[0]), \
                (self.position[1] - edge[1]))

        use_negative = False
        if self.position[0] < edge[0]:
            use_negative = True

        perp_vec = vec2((self.position[0] - edge[0]), \
                (self.position[1] - edge[1]))
        perp_vec.normalize()
        if step == 1:
            if use_negative:
                perp_vec *= -self.source_radius
                perp_vec.rotate(-(math.pi*2/4))
            else:
                perp_vec.rotate((math.pi*2/4))
                perp_vec *= self.source_radius
        else:
            if use_negative:
                perp_vec *= -self.source_radius
                perp_vec.rotate((math.pi*2/4))
            else:
                perp_vec *= self.source_radius
                perp_vec.rotate(-(math.pi*2/4))

        cv = vec2((self.position[0] + perp_vec[0]) - edge[0], (self.position[1] + perp_vec[1]) - edge[1])
        cv *= -1

        cv.normalize()
        return cv * self.radius * 10

    def _render_source(self):
        glBegin(GL_TRIANGLE_FAN)

        # Color
        glColor4f(1.0, 1.0, 0.0, 1.0)
        glVertex3f(self.position[0], self.position[1], self.depth)

        angle = 0
        while angle <= math.pi * 2:
            glVertex3f(self.source_radius * math.cos(angle) + self.position[0],
                    self.source_radius * math.sin(angle) + self.position[1],
                    self.depth)
            angle += math.pi * 2 / 12
        glVertex3f(self.position[0] + self.source_radius, self.position[1],
                self.depth)

        glEnd()

    def render(self, intensity):
        shader = get_shader()
        shader.enable()
        shader.set_state(self.color + [intensity])

        glPushMatrix()
        glTranslate(self.position[0], self.position[1], 0)
        glScale(self.radius, self.radius, 0)
        glBegin(GL_QUADS)
        glVertex3f(-1.0, -1.0, self.depth)
        glVertex3f(1.0, -1.0, self.depth)
        glVertex3f(1.0, 1.0, self.depth)
        glVertex3f(-1.0, 1.0, self.depth)
        glEnd()
        glPopMatrix()
        
        shader.disable()

        # Remove this!
        self._render_source()
