from __future__ import with_statement
from __future__ import absolute_import
from __future__ import division


from shaderobjects import Program


vertex_shader = """
varying vec2 pos;

void main()
{
    pos = gl_Vertex.xy;
    gl_Position = gl_ModelViewProjectionMatrix * vec4(gl_Vertex.xy, 0.0, 1.0);
}
"""
fragment_shader = """
varying vec2 pos;
uniform vec4 color;

void main() {
    float t = 1.0 - sqrt(pos.x*pos.x + pos.y*pos.y);
    // Enable this line if you want sigmoid function on the light interpolation
    //t = 1.0 / (1.0 + exp(-(t*12.0 - 6.0)));
    gl_FragColor = vec4(color.r, color.g, color.b, color.a) * t;
}
"""


class LightShader(object):
    def __init__(self):
        self.program = None

    def create_program(self):
        # Create the program
        self.program = Program(vertex_shader,
                fragment_shader)
                
    def enable(self):
        if not self.program:
            self.create_program()
        self.program.enable()

    def set_state(self, color):
        self.program.set_uniform("color", color, "4f")

    def disable(self):
        self.program.disable()

