from __future__ import with_statement
from __future__ import absolute_import
from __future__ import division


from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *

window = 0
width = 800
height = 600

from light import Light
from convexhull import ConvexHull
from framebufferobject import FrameBufferObject

fbo = None

addlight = True
intensity = 1.0

lights = []
hulls = []

def resize(w, h):
    print w, h
    global width, height, fbo
    width, height = w, h
    glViewport(0, 0, w, h)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    glOrtho(0, w, 0, h, -1, 1)
    
    if not fbo:
        fbo = FrameBufferObject()


def init_gl_scene():
    global lights, hulls, bgtex
    lights.append(Light((0, 300)))

    hulls.append(ConvexHull((150, 200), points=[(0, 0), (50, -75), (100, -75), (150, 0), (100, 75), (50, 75)]))

    for x in xrange(16):
        hulls.append(ConvexHull((100 + x * 40, 500), points=[(0, 0), (20, 0), (20, 20), (0, 20)]))


def clear_framebuffer_alpha():
    glColorMask(0, 0, 0, 1)
    glClear(GL_COLOR_BUFFER_BIT)


def mousemove(x, y):
    y = height - y
    lights[0].set_position((x, y))


def mouseclick(button, state, x, y):
    y = height - y
    if state == GLUT_DOWN:
        if button == GLUT_LEFT_BUTTON:
            lights.insert(0, Light((x, y)))
        elif button == GLUT_RIGHT_BUTTON:
            if len(lights) > 1:
                lights.pop(1)
        print len(lights)


def keyboard(keycode, x, y):
    global addlight, intensity
    if keycode == ' ':
        addlight = not addlight
    elif keycode == '+':
        if intensity <= 0.9:
            intensity += 0.1
    elif keycode == '-':
        if intensity > 0.1:
            intensity -= 0.1
    elif keycode == 'a':
        hulls[0].depth -= 0.1
        print hulls[0].depth
    elif keycode == 'd':
        hulls[0].depth += 0.1
        print hulls[0].depth


def render():
    # Clear the color buffer
    glClearColor(0.0, 0.0, 0.0, 0.0)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT)

    # Use less-than or equal depth testing
    glDepthFunc(GL_LEQUAL)

    fbo.enable()

    # Clear the fbo, and z-buffer
    glClearDepth(1.1)
    glClearColor(0.0, 0.0, 0.0, 0.0)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT)

    # fill z-buffer
    glEnable(GL_DEPTH_TEST)
    glDepthMask(True)
    glColorMask(0, 0, 0, 0)
    for hull in hulls:
        hull.render()
    glDepthMask(False)
    glDisable(GL_DEPTH_TEST)

    for light in lights:
        # Clear the alpha channel of the framebuffer to 0.0
        clear_framebuffer_alpha()


        # Write new framebuffer alpha
        glDisable(GL_BLEND)
        glEnable(GL_DEPTH_TEST)
        glColorMask(0, 0, 0, 1)
        light.render(intensity)

        glEnable(GL_BLEND)
        glBlendFunc(GL_DST_ALPHA, GL_ZERO)
        # Draw shadow geometry
        for hull in hulls:
            hull.draw_shadow_geometry(light)
        glDisable(GL_DEPTH_TEST)

        # Draw geometry
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_BLEND)
        glBlendFunc(GL_DST_ALPHA, GL_ONE)
        glColorMask(1, 1, 1, 0)
        if addlight:
            for light in lights:
                light.render(intensity)
        for hull in hulls:
            hull.render()        

    fbo.disable()

    glDisable(GL_DEPTH_TEST)
    glDisable(GL_BLEND)

    # Render the fbo on top of the color buffer
    fbo.render()

    glutSwapBuffers()


def main():
    global window
    glutInit()
     #glutInitDisplayString( "rgba double samples>=4 ");
    #glEnable(GL_MULTISAMPLE);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH)
    glutInitWindowSize(width, height)
    glutInitWindowPosition(0, 0)
    window = glutCreateWindow("Dynamic 2D Soft Shadows")
        
    glutDisplayFunc(render)
    glutIdleFunc(render)
    glutReshapeFunc(resize)
    glutPassiveMotionFunc(mousemove)
    glutMouseFunc(mouseclick)
    glutKeyboardFunc(keyboard)

	# Initialize our window.
    resize(width, height)
    
    # Create our scene
    init_gl_scene()

    # Start Event Processing Engine
    glutMainLoop()


if __name__ == '__main__':
    main()