from __future__ import with_statement
from __future__ import absolute_import
from __future__ import division


from shaderobjects import Program


vertex_shader = """
varying vec2 pos;

void main()
{
    pos = gl_Vertex.xy;
    gl_Position = gl_ModelViewProjectionMatrix * vec4(gl_Vertex.xy, 0.0, 1.0);
}
"""
fragment_shader = """
uniform vec2 origin;
uniform vec2 inner;
uniform float angle;
uniform float inner_intensity;
uniform float outer_intensity;

varying vec2 pos;

void main()
{
    float a = acos(dot(normalize(pos - origin), normalize(inner))) / angle;
    a = (outer_intensity - inner_intensity) * a + inner_intensity;
    a = 1.0 / (1.0 + exp(-(a*12.0 - 6.0)));
    gl_FragColor = vec4(a, a, a, a);
}
"""


class PenumbraShader(object):
    def __init__(self):
        self.program = None

    def create_program(self):
        # Create the program
        self.program = Program(vertex_shader,
                fragment_shader)

    def enable(self):
        if not self.program:
            self.create_program()
        self.program.enable()

    def set_state(self, origin, angle, inner_vector, inner_intensity,
            outer_intensity):
        self.program.set_uniform("origin", (origin[0], origin[1]), "2f")
        self.program.set_uniform("angle", angle, "1f")
        self.program.set_uniform("inner", (inner_vector[0], inner_vector[1]), "2f")
        self.program.set_uniform("inner_intensity", inner_intensity, "1f")
        self.program.set_uniform("outer_intensity", outer_intensity, "1f")

    def disable(self):
        self.program.disable()
