from __future__ import with_statement
from __future__ import absolute_import
from __future__ import division

import math
import vector2d
from penumbrashader import PenumbraShader

from OpenGL.GL import *


shader = None
def get_shader():
    global shader
    if shader:
        return shader
    shader = PenumbraShader()
    shader.create_program()
    return shader


class ShadowFin():
    def __init__(self, root_position):
        self.root_position = root_position
        self.outer = None
        self.penumbra_intensity = 1.0
        self.inner = None
        self.umbra_intensity = 0.0
        self.depth = 0.0

        self.index = None

    def set_outer(self, vector):
        self.outer = vector

    def set_inner(self, vector):
        self.inner = vector

    def set_umbra_intensity(self, intensity):
        self.umbra_intensity = intensity

    def set_penumbra_intensity(self, intensity):
        self.penumbra_intensity = intensity

    def angle(self):
        uv = self.inner.copy()
        pv = self.outer.copy()
        uv.normalize()
        pv.normalize()
        return math.acos(vector2d.dot(uv, pv))

    def render(self):
        shader = get_shader()
        shader.enable()
        shader.set_state(self.root_position, self.angle(), self.inner,
                self.umbra_intensity,
                self.penumbra_intensity)
        glBegin(GL_TRIANGLES)
        glVertex3f(self.root_position[0], self.root_position[1], self.depth)
        glVertex3f(self.root_position[0] + self.outer[0],
                self.root_position[1] + self.outer[1], self.depth)
        glVertex3f(self.root_position[0] + self.inner[0],
                self.root_position[1] + self.inner[1], self.depth)
        glEnd()
        shader.disable()
