from __future__ import with_statement
from __future__ import absolute_import
from __future__ import division


from math import *


class Vector2D(object):
    def __init__(self, x_or_tuple, y = None):
        if y is None:
            self.x, self.y = x_or_tuple            
        else:
            self.x = x_or_tuple
            self.y = y

    def __repr__(self):
        return "<Vector2D " + str((self.x, self.y)) + ">"
    
    def __getitem__(self, index):
        if index == 0:
            return self.x
        elif index == 1:
            return self.y
        else:
            raise IndexError ('value index out of range')
        
    def __setitem__(self, index, value):
        if index == 0:
            self.x = value
        elif index == 1:
            self.y = value
        else:
            raise IndexError ('value index out of range')
    
    def __eq__(self, other):
        if self.x == other[0] and \
                self.y == other[1]:
            return True
        return False
    
    def __ne__(self, other):
        return not self == other
    
    def __nonzero__(self):
        if self.x == 0 and self.y == 0:
            return False
        else:
            return True
    
    def __add__(self, other):
        try:
            x = self.x + other[0]
            y = self.y + other[1]
        except IndexError, TypeError:
            raise NotImplementedError('non-legal operation: addition with '
                    + other)
        return Vector2D(x, y)

    def __radd__ (self, other):
        return self.__add__(other)
    
    def __sub__(self, other):
        try:
            x = self.x - other[0]
            y = self.y - other[1]
        except IndexError, TypeError:
            raise NotImplementedError('non-legal operation: subtraction with '
                    + str (other))
        return Vector2D(x, y)

    def __rsub__(self, other):
        return self.__sub__(other) * -1
            
    def __mul__(self, other): 
        try:
            other * 2 + 1
        except:
            raise NotImplementedError('non-legal operation: multiplication '\
                    'with ' + str(other))
        return Vector2D(self.x * other, self.y * other)
    
    def __rmul__(self, other):
        return self.__mul__ (other)

    def copy(self):
        return Vector2D(self.x, self.y)
    
    def rotate(self, rad):
        """Rotate vector with radians."""
        length = self.length()
        if self.y != 0:
            rad += asin(self.y/length)
        elif self.x != 0:
            rad += acos(self.x/length)
        else:
            # Null Vector
            return
        self.x = length * cos(rad)
        self.y = length * sin(rad)
    
    def normalize(self):
        length = self.length()
        self.x /= length
        self.y /= length
    
    def length(self):
        return sqrt(self.x**2 + self.y**2)

    def angle(self):
        return atan2(self.y, self.x)


def dot(a, b):
    return a[0] * b[0] + a[1] * b[1]

def cross(a, b):
    return a[0] * b[1] - a[1] * b[0]

def from_points(from_point, to_point):
    return Vector2D(from_point[0] - to_point[0], from_point[1] - to_point[1])